{% extends base %}

{% block title %}Roentgen X-ray Interaction with Matter{% endblock %}

{% block postamble %}
  <style>
    {% include 'styles.css' %}
  </style>
  <!-- Global site tag (gtag.js) - Google Analytics -->
  <script async src="https://www.googletagmanager.com/gtag/js?id=UA-149262170-1"></script>
  <script>
    window.dataLayer = window.dataLayer || [];
    function gtag(){dataLayer.push(arguments);}
    gtag('js', new Date());

    gtag('config', 'UA-149262170-1');
  </script>
{% endblock %}

{% block contents %}
  <div class="content">
    <div id="header" style="border-radius: 10px; background-color: darkgray; padding: 1%;">
          <img src="https://raw.githubusercontent.com/ehsteve/roentgen/master/docs/logo/roentgen_logo.svg" height="100" width="100" style="border-radius: 50%; display: inline-block"/>
          <h1 style="display: inline-block">Roentgen X-ray Interaction with Matter</h1>
    </div>
    <p>Calculates the transmission of x-rays through various materials. The
      following links provide a list of all available <a href="https://github.com/ehsteve/roentgen/blob/master/roentgen/data/elements.csv">elements and their default densities</a>
      as well as <a href="https://github.com/ehsteve/roentgen/blob/master/roentgen/data/compounds_mixtures.csv">advanced materials</a>.
    </p>

    {{ super() }}

 </div>

<hr>
<footer class="footer">
    <p style="text-align: center">
      Data provided by <a href="https://www.nist.gov/pml/x-ray-mass-attenuation-coefficients">NIST</a> | Powered by <a href="https://github.com/ehsteve/roentgen">Roentgen</a>, <a href="https://bokeh.pydata.org/en/latest/">Bokeh</a>, and <a href="https://www.astropy.org">Astropy</a><br>
      A comparison with the <a href="http://henke.lbl.gov/optical_constants/">CXRO</a> can be found <a href="https://roentgen.readthedocs.io/en/latest/guide/cxro_compare.html">here</a> |  Edit this app on <a href="https://github.com/ehsteve/roentgen/tree/master/gui">Github</a><br>
    </p>
</footer>
<br>
{% endblock %}