def get_package_data():
    return {
        "roentgen": [
            "data/*.json",
            "data/*.csv",
            "data/elements/*.csv",
            "data/compounds_mixtures/*.csv",
        ]
    }
